<?php
Namespace WordPress\Plugin\Encyclopedia;

class Updates {
  public
    $plugin_file, # absolute path to the main file of the plugin
    $username, # The username of the subscriber
    $password, # The password of the subscriber
    $show_notification = True; # Show update notifications to the user or not

  function __construct($plugin_file, $username = Null, $password = Null, $show_notification = True){
    # Collect parameters
    $this->plugin_file = $plugin_file;
    $this->username = $username;
    $this->password = $password;
    $this->show_notification = $show_notification;

    # Get base directory url
    $this->base_url = get_bloginfo('wpurl').'/'.SubStr(RealPath(DirName($this->plugin_file)), Strlen(ABSPATH));

    # Set Filters
    Add_Filter('pre_set_site_transient_update_plugins', Array($this, 'Filter_Update_Plugins'));
    Add_Filter('plugins_api', Array($this, 'Filter_Plugins_API'), 10, 3);
  }

  function Phone_Home($endpoint, $purpose, $parameter = Array()){
    $parameter = Is_Array($parameter) ? $parameter : Array();
    $parameter = Array_Merge($parameter, Array(
      'purpose' => $purpose,
      'format' => 'json',
      'subscriber' => RAWUrlEncode($this->username),
      'locale' => Get_Locale(),
      'referrer' => RAWUrlEncode(Home_Url())
    ));
    $url = Add_Query_Arg($parameter, $endpoint);
    $raw_response = @WP_Remote_Get($url, Array('timeout' => 5));
    If (!$raw_response || Is_WP_Error($raw_response)) return False;
    $response = @JSON_Decode($raw_response['body']);
    return $response;
  }

  function Filter_Update_Plugins($value){
    If (!Function_Exists('Get_Plugins')) return $value;

    # Find this plugin
    $found_plugin = False;
    ForEach ( (Array) Get_Plugins() AS $file => $data ){
      If (SubStr($this->plugin_file, -1*StrLen($file)) == $file){
        $plugin_file = $file;
        $plugin_data = $data;
        $found_plugin = True;
        Break;
      }
    }
    If (!$found_plugin) return $value;

    # Get current version from server
    $remote_page = $this->Phone_Home($plugin_data['PluginURI'], 'version_check');
    If (!$remote_page) return $value;

    # Check if the update function is disabled
    If (!$this->show_notification) return $value;

    # Compare versions
    If (Version_Compare($plugin_data['Version'], $remote_page->version, '<')){
      $value->response[$plugin_file] = (Object) Array(
        'id' => $remote_page->id,
        'slug' => BaseName($this->plugin_file),
        'new_version' => $remote_page->version,
        'url' => $remote_page->url,
        'package' => SPrintF($remote_page->download, RAWUrlEncode($this->username), RAWUrlEncode($this->password))
      );
    }

    # Return the filter input
    return $value;
  }

  function Filter_Plugins_API($false, $action, $args){
    If ($action == 'plugin_information' && $args->slug == BaseName($this->plugin_file)){
      WP_Enqueue_Style('plugin-details', $this->base_url . '/plugin-details.css' );
      $plugin_data = Get_Plugin_Data($this->plugin_file);
      $plugin_data = $this->Phone_Home($plugin_data['PluginURI'], 'get_plugin_details');
      $plugin = (Object) Array(
        'name' => $plugin_data->name,
        'slug' => BaseName($this->plugin_file),
        'version' => $plugin_data->version,
        'author' => SPrintF('<a href="%1$s">%2$s</a>', $plugin_data->author->url, $plugin_data->author->display_name),
        'author_profile' => $plugin_data->author->url,
        'contributors' => Array( 'dhoppe' => $plugin_data->author->url ),
        'requires' => '3.8',
        'tested' => Date('y.n.j'),
        'rating' => Round(Rand(90, 100)),
        'num_ratings' => Round( (Time() - 1262300400) / (3*24*60*60)),
        'downloaded' => Round( (Time() -  1262300400) / 600 ),
        'last_updated' => Date('Y-m-d', Time() - (1 * 24 * 3600) ),
        'homepage' => $plugin_data->url,
        'download_link' => SPrintF($plugin_data->download, RAWUrlEncode($this->username), RAWUrlEncode($this->password)),
        'sections' => Is_Object($plugin_data->content) ? (Array) $plugin_data->content : Array( __('Description') => (String) $plugin_data->content ),
        'external' => True
      );
      return $plugin;
    }
    Else return $false;
  }

}